;;; pmwiki.el --- edit PmWiki pages within Emacs
;;
;; Copyright (C) 2003  Christian Ridderstrm
;;
;; Author:  Christian Ridderstrm <christian.ridderstrom@home.se>
;; Maintainer: Christian Ridderstrm <christian.ridderstrom@home.se>
;; Release: $Name: release-1_2 $
;; Revision: $Revision: 1.11 $
;; Keywords: hypermedia
;; URL: http://www.emacswiki.org/cgi-bin/wiki.pl?PmWikiEditMode
;;
;; 
;;
;; This file is not part of GNU Emacs.
;;
;;; License:
;;
;; This is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.
;;
;;; Commentary:
;;
;; Background:
;;
;; `pmwiki-mode' was created because in order to easily edit wiki
;; pages at http://wiki.lyx.org (also see http://www.lyx.org). This
;; site the wiki engine PmWiki (see http://www.pmichaud.com/wiki).
;;
;; `pmwiki-mode' is based on the source of `simple-wiki-mode', i.e.
;; 	simple-emacswiki.el, simple-wiki-edit.el and simple-wiki.el.
;; `simple-wiki-mode' was written by Alex Schroeder <alex@gnu.org>
;; (see http://www.emacswiki.org/cgi-bin/wiki.pl/SimpleWikiEditMode).
;;
;;
;; Requirements:
;;
;; This file has been tested with Emacs 21.2.1 and requires these
;; additional packages:
;;	`http-post'	http://www.emacswiki.org/cgi-bin/wiki.pl/HttpPost
;;	`http-get'	http://www.emacswiki.org/cgi-bin/wiki.pl/HttpGet
;;
;; In addition, this package is useful:
;;	`longlines'	http://www.emacswiki.org/cgi-bin/wiki.pl/LongLines
;;
;; If you downloaded this file as a tar.gz-file, you should already
;; have these packages. Note that the version of `longlines' in the
;; archive has been patched --- this line has been added to the end:
;;	(provide 'longlines)
;;
;;
;; Installation:
;;
;; 1. Add these lines to your .emacs-file:
;;
;;	(add-to-list 'load-path "~/emacs/pmwiki")
;;	(require 'pmwiki)
;;
;;    where it is assumed that this file is located in the directory:
;;
;;	~/emacs/pmwiki
;;
;; 2. Optionally, compile this file with the command `byte-compile'.
;; 3. Restart Emacs.
;;
;;
;; Usage:
;;
;; Two examples to get you started:
;; * Use the command `pmichaud-open' with the argument
;;	Main.WikiSandbox
;;   to download the page Main.WikiSandbox from the PmWiki wiki.
;;
;; * Use the command `pmwiki-open' to open a page by specifying
;;   its URL. For example, to open the page where this mode is
;;   described, give the following URL as argument to the command:
;;
;;	http://www.pmichaud.com/wiki/Cookbook/EmacsPmWikiMode
;;
;; Enjoy. /Christian
;;
;;
;; Configuration options:
;;
;; Note that the code below is meant to be inserted in .emacs (or an
;; equivalent file).
;; 
;; If you want to use any of the configuration examples below, first
;; insert this line of code:
;;
;;	(defvar pmwiki-mode-hooks)
;;
;; If your auto-fill-mode is usually on, you might want to turn it off:
;;
;;	(add-hook 'pmwiki-mode-hooks 'turn-off-auto-fill)
;;	
;;
;; Also consider using the package longlines.el, for details see
;;   http://www.emacswiki.org/cgi-bin/wiki.pl/LongLines 
;;
;; 	(add-hook 'pmwiki-mode-hooks
;;	  'pmwiki-delayed-hook-longlines-mode-on)
;;	(add-hook 'pmwiki-save-before-hooks 'longlines-mode-off)
;;
;; in which case, also customize `pmwiki-fill-column' back to 70.
;; Note that you may have to patch the file longlines.el by adding
;; 	(provide 'longlines)
;; to the end of it. As an alternative configuration, this code also
;; makes the `longlines-mode' show the hard newlines.
;;
;; 	(add-hook 'pmwiki-mode-hooks
;;	  'pmwiki-delayed-hook-longlines-mode-showing-hard-newlines)
;;	(add-hook 'pmwiki-save-before-hooks 'longlines-mode-off)
;;
;;
;;
;; Note: PmWiki supports using '\' as line continuation character,
;; i.e. if it is the the last character on the line the linebreak is
;; ignored. This is not supported.
;;

;;; Code:
(require 'thingatpt)
(require 'http-get)
(require 'http-post)

;; ----------------------------------------------------------------------
;;
;;	Preliminaries
;;
;; Define variables with various patterns
;;
(defvar pmwiki-GroupNamePattern "[A-Z][A-Za-z0-9]+"
  "Pattern used to define a the name of a group of a PmWiki-page")

(defvar pmwiki-WikiWordPattern
  "[A-Z][A-Za-z0-9]*\\(?:[A-Z][a-z0-9]\\|[a-z0-9][A-Z]\\)[A-Za-z0-9]*"
  "Pattern that defines a WikiWiki-word in a PmWiki-page.")

(defvar pmwiki-PageTitlePattern
  "[A-Z][A-Za-z0-9]*\\(?:-[A-Za-z0-9]+\\)*"
  "Pattern for a page name in an URL.")

;; ...(<GroupName>)[/.](<PageTitle>)(?|$)
(defvar pmwiki-url-pattern
  (concat "^\\(https?:.+\\)/"			; Match 1
	  "\\(" pmwiki-GroupNamePattern "\\)/"	; Match 2
	  "\\(" pmwiki-PageTitlePattern "\\)" 	; Match 3
	  "\\(?:\\?\\|$\\)" )
  "This pattern is used on `pmwiki-page-url' to extract various parts.
The parts matches \"<BaseURL>/<group>/<page>?\". where
 1 - <BaseURL>, 2 - <group> and 3 - <page>.
The parts of the pattern are derived from `pmwiki-GroupNamePattern'
and `pmwiki-PageTitlePattern'.")
	     
(defvar pmwiki-FragmentPattern
  "#[A-Za-z][-.:\\w]*"
  "Pattern that defines an appended #-target to a page.")

;; Link patterns:
;; 	\b(<GroupName>[/.])?<FreeLink><Fragment>
;;	<FreeLink><Fragment>?
;;	\b<GroupName>[/.]<WikiWord><Fragment>?
;;	\[\[#([A-Za-z][-.:\w]*?\]\]		; que?
;;	\b<WikiWord><Fragment>?

;; Pattern of WikiWiki-link: [<GroupName>{.|/}]<WikiWord>
(defvar pmwiki-link-pattern
  (concat 
   "\\<"
   "\\(?:\\(" pmwiki-GroupNamePattern "\\)[./]\\)?" 
   "\\(" pmwiki-WikiWordPattern "\\)" )
  "The pattern used for finding WikiWiki-words.")

;; Pattern of a free link (from .php-file)
;; "{{(?>"
;;    "([A-Za-z][A-Za-z0-9]*"
;;	"(?:"
;;	    "(?:[\\s_]*|-)"
;;	    "[A-Za-z0-9]+"
;;      ")*"
;;    ")"
;;	"(?:\\|((?:(?:[\\s_]*|-)[A-Za-z0-9])*))?"
;; ")}}"
;;    "((?:-?[A-Za-z0-9]+)*)";
(defvar pmwiki-FreeLinkPattern
  (concat 
   "{{\\(\\([A-Za-z][A-Za-z0-9]*"
   "\\(?:\\(?:[	 _]*\\|-\\)[A-Za-z0-9]+\\)*\\)"
   "\\(?:|\\(\\(?:\\(?:[ 	_]*\\|-\\)[A-Za-z0-9]\\)*\\)\\)?\\)}}"
   "\\(\\(?:-?[A-Za-z0-9]+\\)*\\)")
  "Pattern of a {{free link}} in PmWiki-page")

;;
;; Other patterns
;;
(defvar pmwiki-StylePattern
  "%%\\|%[A-Za-z][-,=#\\w\\s]*%"
  "Pattern for PmWiki style tags.")

;; ----------------------------------------------------------------------
;;
;; Define variables with some default URL's
;;

(defvar pmichaud-sandbox-url "http://www.pmichaud.com/wiki/Main/WikiSandbox"
  "URL to sandbox at http://www.pmichaud.com")

(defvar pmwiki-default-url pmichaud-sandbox-url
  "Default URL used by some interactive functions")

(defvar pmwiki-open-url-default pmichaud-sandbox-url
  "Default URL used by `pmwiki-open-url'.")

(defvar lyx-wiki-base-url "http://wiki.lyx.org/pmwiki.php"
  "Base URL of the Lyx Wiki site")

(defvar lyx-homepage-url (concat lyx-wiki-base-url "/Main/HomePage")
  "URL for the home page of the LyX wiki")

;; ----------------------------------------------------------------------
;;
;; Define general variables 
;;
;; `pmwiki-page-url' becomes local to a buffer in `pmwiki-mode'
(defvar pmwiki-page-url nil
  "The URL of the page corresponding to the current buffer, thus defining
the base URL of the wiki engine as well as group and page name.")

;;
;; There are too many "URL's" now... basically we have
;; 'base URL' -> the PmWiki-script, e.g. http://wiki.lyx.org/pmwiki.php
;; 'post URL' -> to where the form with a page is posted, same as 'base URL'
;; 'page URL' -> the page, e.g. http://wiki.lyx.org/pmwiki.php/Main/WikiWord
;; 'source URL' -> source of PmWiki-page, e.g.
;;		http://wiki.lyx.org/pmwiki.php/Main/WikiWord?action=source
;;
;; I'm thinking of storing the location data as a list, location-list
;; This requires conversion methods:
;;	data    -> pageURL	=> (pmwiki-loc 'URL  listData)
;;	        -> baseURL	=> (pmwiki-loc 'base listData)
;;	pageURL -> data		=> (pmwiki-url-to-loc-list URL)
;;	srcURL  -> data		=> (pmwiki-url-to-loc-list URL)
;;(defvar pmwiki-location-list nil
;;  "List with three strings, (<baseURL> <group> <page>), specifying
;;the base URL of the wiki engine as well as a group name and a page name.")
;;

(defvar pmwiki-save-function 'pmwiki-default-save-function
  "Specifies the function used to save a PmWiki-buffer.")

(defcustom pmwiki-fill-column 70 "Fill column width for `pmwiki-mode'.")
(defcustom pmwiki-mode-hooks nil "")
(defcustom pmwiki-save-before-hooks nil "")

;; ----------------------------------------------------------------------
;;
;;	Auxiliary helper function
;;
(defun pmwiki-default-string (s s0)
  "Return S if not empty, otherwise return S0"
  (if (string= s "") s0 s))

(defun pmwiki-name-is-url (name)
  "Checks if NAME begins with 'http:' or 'https:'"
  (string-match "^https?:" name))

;; ----------------------------------------------------------------------
;;
;;	Main code
;;
;; Define the `pmwiki-mode'. 
;;
;; Have to modify patterns for faces. List of faces:
;; builtin, comment, constant, doc, function, keyword, string, type, variable
;; warning
;;	[=...=]			- string
;;	[[#<target>]]		- constant
;;	[[<url> <text>]]	- type
;;	{{link}}		- keyword
;;	Wiki-links		- keyword
;;	 <monospace>		- comment
;;	@@<monospace>@@		- comment
;;	''<emp>''		- type
;;	'''<strong>'''		- function
;;	'''''<extra>'''''	- warning
;;	*...			- builtin
;;	#...			- doc
;;	!...			- function
;;	:..: : <indented par.>	- doc
;;
;; Maybe remove sgml-tag-alist?

;; Special faces
(defface pmwiki-question-face
  '((t (:bold t)))
  "Used to indicate a question in a PmWiki page.")

(defface pmwiki-answer-face
  '((t (:bold t :italic t)))
  "Used to indicate an answer in a PmWiki page.")

(define-derived-mode pmwiki-mode text-mode "PmWiki"
  "Mode to edit PmWiki pages. Typically you start by using the
function `pmwiki-open' to open a URL, e.g.
	http://www.pmichaud.com/wiki/Main/HomePage

Alternatively, the same page can be opened with `pmichaud-open' by
giving 'Main.HomePage' or 'Main/HomePage' as argument. Similarly, a
page at the LyX wiki can be opened using `lyx-open'.

Once a page has been opened, you can edit it and then save it using
`pmwiki-save' or `pmwiki-save-as'. The latter function allows you to
save the page with a new name, and even on a different wiki engine by
giving an URL as argument.

To view the wiki page that you are editing in a browser, first save it
and then use the command `pmwiki-browse'.

While editing the page, you can use `pmwiki-next' and `pmwiki-prev' to
quickly go to the next and previous links. And when the point is on a
link, you can use `pmwiki-follow' to goto the page that the link
points to, i.e. that page is opened in a new buffer.

Interactive functions:
`pmwiki-open'    - Open wiki page specified by group/name or URL
`pmwiki-next'    - Goto the next WikiWord or {{free link}}.
`pmwiki-prev'    - Goto the previous WikiWord or {{free link}}.
`pmwiki-follow'  - Open (goto) page pointed to by link at point.
`pmwiki-save'    - Save current buffer to the wiki engine.
`pmwiki-save-as' - Save buffer to new page specified by group/name or URL.
`pmwiki-browse'  - Show current buffer in browser.

`pmichaud-open'  - Open page from the Pmichaud wiki.
`lyx-open'       - Open a page from the LyX wiki.
`lyx-save-as'    - Save current buffer to page at the LyX wiki.

\\{pmwiki-mode-map}"
  (font-lock-add-keywords
   nil
   '(
     ("^[ \t].+?$" . font-lock-comment-face)		;  <monospace>
     ("\\[=[^=]*=\\]" . font-lock-function-name-face)	; [= ... =]
     ("@@[^ \t@][^@]*[^ \t@]@@" . font-lock-comment-face) ; @@a .. b@@
     ("\\(^[*#]+\\)[^*#]" 1 font-lock-constant-face)	; {*|#}+ ...
     ("^!+[^!].*" . font-lock-constant-face)		; !+ ...
     ("\\[\\[<<\\]\\]" . font-lock-comment-face)	; [[<<]]
     ("\\[\\[#[^]]+\\]\\]" . font-lock-constant-face)	; [[#<target>]]
     ("\\\\$" . font-lock-keyword-face)			;     \
     ("\\\[\[[^#].*\]\]" . font-lock-type-face)		; [[ ... ]]
     ("{{[^}]*}}\\(\\(?:-?[A-Za-z0-9]+\\)*\\)" . font-lock-keyword-face)
     ("^:+ +:" . font-lock-doc-face)			; :..: : 
     ("^:[ \t]*[^ \t:]+[ \t]*+:" . font-lock-string-face)	; : term : 
     ("%[^ \t%][^%]*[^ \t%]%" . font-lock-warning-face)	; %color = red%
     ("^Q:" . 'pmwiki-question-face)			; Q:
     ("^A:" . 'pmwiki-answer-face)			; Q:
     ))
   (font-lock-add-keywords
   nil
   (list (cons (symbol-value 'pmwiki-link-pattern) 'font-lock-keyword-face))
   )

  (font-lock-mode 1)
  (goto-address)
  (set (make-local-variable 'sgml-tag-alist)
       `(("b") ("code") ("em") ("i") ("strong") ("nowiki")
	 ("pre" \n) ("tt") ("u")))
  (set (make-local-variable 'skeleton-transformation) 'identity)
  (setq indent-tabs-mode t)
  (run-hooks 'pmwiki-mode-hooks))

(define-key pmwiki-mode-map (kbd "C-c C-c") 'pmwiki-save)
(define-key pmwiki-mode-map (kbd "C-c C-w") 'pmwiki-save-as)
(define-key pmwiki-mode-map (kbd "C-c C-b") 'pmwiki-browse)
(define-key pmwiki-mode-map (kbd "C-c C-o") 'pmwiki-open)
(define-key pmwiki-mode-map (kbd "C-c C-f") 'pmwiki-follow)
(define-key pmwiki-mode-map (kbd "C-c C-g") 'pmwiki-follow)
(define-key pmwiki-mode-map (kbd "C-c C-n") 'pmwiki-next)
(define-key pmwiki-mode-map (kbd "C-c C-p") 'pmwiki-prev)


;; ----------------------------------------------------------------------
;;
;; `pmwiki-edit' - main function used to load a PmWiki-page into a buffer
;;
;;
;;;###autoload
(defun pmwiki-edit (link &optional base-url goto-end http-ver content-type)
  "Download the wiki page specified by LINK and BASE-URL.
If BASE-URL is not given, it's taken from `pmwiki-page-url'.
Then change the buffer's mode to `pmwiki-mode'.
If GOTO-END is nil, wait until the process is done and move point to
the beginning of the buffer."
  (unless content-type (setq content-type 'iso-8859-1))
  (let ((page-url (pmwiki-create-page-url link base-url))
	(proc nil))
    (set 'proc
	 (http-get (concat page-url "?action=source")	; Source URL
		   '(("Connection" "close"))		; headers
		   nil					; ?
		   http-ver				; nil -> 1.0
		   (pmwiki-link-to-name link)		; output buffer
		   content-type))			; typically nil
    (pmwiki-mode)
    (set-fill-column pmwiki-fill-column)
    (set (make-local-variable 'pmwiki-page-url) page-url)
    (unless goto-end
      (while (string-equal (process-status proc) "open") 
	(sleep-for 0 50))
      (goto-char 1))
    proc))


(defun pmwiki-loc (P &optional url)
  "Extract BaseURL, GroupName or PageName from URL.
If URL is not given, `pmwiki-page-url' is used instead.
`pmwiki-url-pattern' is used to match to different parts of URL:
	URL = \"<BaseURL>/<group>/<page>?...\"
where part, P, determines what is returned according to this list:
	1 or 'base	-> <BaseURL>
	2 or 'group	-> <group>
	3 or 'page	-> <page>
	     'link	-> (<group> . <page>).
	     'name      -> \"<group>.<page>\"
"
  (unless url (set 'url pmwiki-page-url))
  (unless url (error "No URL given, set `pmwiki-page-url' first!"))
  (if (numberp P)
      (let ((case-fold-search nil))
	(if (string-match pmwiki-url-pattern url)
	    (match-string P url)
	  (error "Cannot determine part %d in %s" P url)))
    (unless (symbolp P) (error "Invalid type of argument"))
    (let ((s (symbol-name P)))
      (if (string-equal s "base") (pmwiki-loc 1 url)
	(if (string-equal s "group") (pmwiki-loc 2 url)
	  (if (string-equal s "page") (pmwiki-loc 3 url)
	    (if (string-equal s "link")
		(cons (pmwiki-loc 2 url) (pmwiki-loc 3 url))
	      (if (string-equal s "name")
		  (pmwiki-link-to-name (pmwiki-loc 'link url))
		(error "Invalid argument 'P' of `pmwiki-loc'!"))
	      )))))))


(defun pmwiki-create-page-url (link &optional base-url)
  "Create URL for a page specified by LINK,  with WikiEngine at BASE-URL.
The parameter link LINK is (cons <group-name> <page-name>).
The parameter BASE-URL is an optional URL to the WikiEngine and
taken from `pmwiki-page-url' if BASE-URL is not given."
  (unless base-url (set 'base-url (pmwiki-loc 'base)))
  (unless base-url
    (error "pmwiki-create-page-url: No suitable BASE-URL available"))
  (unless link (error "pmwiki-create-page-url: No LINK given!"))
  (unless (string-match "^https?:" base-url)
    (error "pmwiki-create-page-url: BASE-URL doesn't start with 'http:'"))
  (if (string-match "\\([^?]*\\)\\(\\?.*\\|/?\\)$"
		    base-url) ; Remove last '/'
      (concat (match-string 1 base-url) "/" (car link) "/" (cdr link))
    (error "Unable to remove last '/' from BASE-URL")))


(defun pmwiki-next ()
  "Goto the next WikiWord or {{free link}}."
  (interactive)
  (let ((case-fold-search nil))
    (re-search-forward
     (concat "\\(" pmwiki-link-pattern "\\|" pmwiki-FreeLinkPattern "\\)")
     nil t)))

(defun pmwiki-prev ()
  "Goto the previous WikiWord or {{free link}}."
  (interactive)
  (let ((case-fold-search nil))
    (re-search-backward
     (concat "\\(" pmwiki-link-pattern "\\|" pmwiki-FreeLinkPattern "\\)")
     nil t)))


(defun pmwiki-follow ()
  "Goto (open) the page pointed to by the PmWiki-link at point."
  (interactive)
  (pmwiki-edit (pmwiki-link-at-point)))


(defun pmwiki-page-name-at-point ()
  "If point is on a page link, return page name. Otherwise, return nil."
  (if (thing-at-point-looking-at
       (concat "\\<\\(" pmwiki-WikiWordPattern "\\)"))
      (match-string 1)
    nil))


(defun pmwiki-page-link-at-point ()
  "If point is on a page link, return link with default group and page name.
Otherwise, return nil."
  (let ((page (pmwiki-page-name-at-point)))
    (if page (cons (pmwiki-loc 'group) page)
      nil)))


(defun pmwiki-link-at-point ()
  "Return PmWiki-link in `cons' (\"<group>\", \"<page>\") at point 
of the current buffer. If no link is found at the current point, nil is
returned."
  (let ((case-fold-search nil)
	(found-full-link (thing-at-point-looking-at
			  "\\<[A-Za-z0-9]+[./][A-Za-z0-9]+\\>")))
    (if found-full-link
	(let ((maybe-link (match-string 0)))
	  (message (concat "maybe-link: " maybe-link))
	  (if (string-match (concat "\\<" "\\(" pmwiki-GroupNamePattern "\\)"
				    "[./]\\(" pmwiki-WikiWordPattern "\\)")
			    maybe-link)
	      (cons (match-string 1 maybe-link) (match-string 2 maybe-link))
	    (pmwiki-page-link-at-point)))
      (pmwiki-page-link-at-point))))


(defun pmwiki-name-to-link (name)
  "Convert a string with a name, e.g. \"Group.Name\" to a link, i.e.
	( \"Group\" . \"Name\" )
The conversion allows names in these formats:
	<Group>.<Name>	-> ( <Group> <Name> )
	<Group>/<Name>	-> ( <Group> <Name> )
	<Name>		-> ( <current group> <Name> )
where <current group> is the group that the current buffer belongs to."
  (when (string-match
	 (concat "^\\(?:\\(" pmwiki-GroupNamePattern "\\)[./]\\)?"
		 "\\(" pmwiki-PageTitlePattern "\\)$")
	 name)
    (let ((group (match-string 1 name))
	  (page (match-string 2 name)))
      (if group (cons group page)
	(cons (pmwiki-loc 'group) page)))))
		

(defun pmwiki-link-to-name (&optional link)
  "Convert a LINK to a string with its name. If LINK is not given, the
data is taken from `pmwiki-page-url'."
  (unless link (set 'link (pmwiki-loc 'link)))
  (concat (car link) "." (cdr link)))


(defun pmwiki-show-link (link)
  "Show a PmWiki LINK, i.e. a pair with a group and page name.
This function is mainly used for testing."
  (message (if link (pmwiki-link-to-name link) "Not a link!")))


(defun pmwiki-buffer-link (&optional buf)
  "Return link for buffer BUF. If BUF is not given, the current buffer
is assumed"
  (unless buf (set 'buf (buffer-name)))
  (save-excursion
    (set-buffer buf)
    (pmwiki-loc 'link)))


(defun pmwiki-buffer-url (&optional buf)
  "Return URL of page in buffer BUF. If BUF is not given, the 
current buffer is assumed."
  (save-excursion
    (if buf (set-buffer buf))
    (pmwiki-create-page-url (pmwiki-buffer-link buf))))


(defun pmwiki-browse (&optional buf)
  "Open the buffer BUF in a browser. If BUF is not given,
the current buffer is used."
  (interactive)
  (browse-url (pmwiki-buffer-url buf)))


(defun pmwiki-browse-link-manual (link &optional base-url)
  "Open page in browser URL specified by LINK, with WikiEngine at BASE-URL.
The parameter link LINK is (cons <group-name> <page-name>).
The parameter BASE-URL is an optional URL to the WikiEngine and
is taken from `pmwiki-page-url' if BASE-URL is not given."
  (browse-url (pmwiki-create-page-url link base-url)))


(defun pmwiki-open (name)
  "Open a page specified by NAME from the current wiki engine, or
open page specified by a URL."
  (interactive "sName or URL (ex: Main.HomePage): ")
  (if (pmwiki-name-is-url name)
      (let ((pmwiki-page-url name))(pmwiki-edit (pmwiki-loc 'link)))
    (when (string-equal name "") (set 'name "Main/HomePage"))
    (unless pmwiki-page-url (error "pmwiki-open: `pmwiki-page-url' is empty!"))
    (pmwiki-edit (pmwiki-name-to-link name))))


(defun pmwiki-revert ()
  "Reload the current page from the wiki engine."
  (interactive)
  (unless pmwiki-page-url (error "pmwiki-revert: `pmwiki-page-url' is empty!"))
  (let ((old-point (point-marker))
	(proc nil))
    (set 'proc (pmwiki-open (pmwiki-loc 'name)))
    (while (string-equal (process-status proc) "open") 
      (sleep-for 0 50))
    (goto-char old-point)))


(defun pmwiki-save ()
  "Save the current buffer to the wiki."
  (interactive)
  (unless pmwiki-save-function (error "No save function specified"))
  (run-hooks 'pmwiki-save-before-hooks)
  (save-excursion (funcall pmwiki-save-function)))


(defun pmwiki-save-to-url (page-url)
  "Save current buffer to a PAGE-URL."
  (set 'pmwiki-page-url page-url)
  (rename-buffer (pmwiki-loc 'name))
  (pmwiki-save))


(defun pmwiki-save-as (name)
  "Save current buffer to a specific page on a wiki. This also
changes `pmwiki-page-url', so future saves also goes to this page.
The argument NAME can be either a name (= <group>.<page>) or an URL.
An URL must begin with 'http'."
  (interactive "sName or URL (ex: Main.WikiSandbox): ")
  (if (string-match "^https?:" name)
      (pmwiki-save-to-url name)
    (pmwiki-save-to-url (pmwiki-create-page-url (pmwiki-name-to-link name)))))
      

(defun lyx-save-as (name)
  "Save buffer to a certain <group>.<page> (=NAME) at the LyX wiki"
  (interactive "sName (ex: Main.WikiSandbox): ")
  (let ((page-url (pmwiki-create-page-url (pmwiki-name-to-link name)
					  lyx-wiki-base-url)))
    (message (concat "Saving to " page-url))
    (pmwiki-save-to-url page-url)))
	

;;
;; Functions useful for configuration.
;;
(defun pmwiki-delayed-hook-longlines-mode-on ()
  "Hook that activates `longlines-mode' when Emacs has been idle for 1 s."
  (require 'longlines)
  (run-with-idle-timer
   1 nil
   (lambda (buf)
     (save-excursion
       (set-buffer buf)
       (message (concat "Enabling longlines-mode for " (buffer-name)))
       (longlines-mode-on)))
   (current-buffer)))

(defun pmwiki-delayed-hook-longlines-mode-showing-hard-newlines ()
  "Hook that activates `longlines-mode' when Emacs has been idle for 1 s,
and in addition runs it `longlines-show-hard-newlines'."
  (require 'longlines)
  (run-with-idle-timer
   1 nil
   (lambda (buf)
     (save-excursion
       (set-buffer buf)
       (message (concat "Enabling longlines-mode for " (buffer-name)))
       (longlines-mode-on)
       (longlines-show-hard-newlines)))
   (current-buffer)))


;;; ************************************************************************
;;;
;;; Based on simple-emacswiki.el
;;;

(defun pmwiki-default-save-function ()
  "Save the current page to a PmWiki wiki."
  (let ((cur-buffer (buffer-name))
	(page-url pmwiki-page-url)
	(pagename (pmwiki-loc 'name)))
    (message (concat "Any output is in "
		   (buffer-name 
		    (process-buffer
		     (http-post
		      (pmwiki-loc 'base) ;; Get destination
		      (list (cons "pagename" pagename)
			    (cons "action" "post")
			    (cons "text" (buffer-string)))
		      'iso-8859-1)))))
    (switch-to-buffer cur-buffer)
    (pmwiki-mode)
    (set (make-local-variable 'pmwiki-page-url) page-url)))


(defun pmichaud-open (name)
  "Browse the PmWiki Wiki. Ex: 'Main.WikiSandbox' will open the page from
	http://www.pmichaud.com/wiki/Main/WikiSandbox"
  (interactive "sName (default Main.WikiSandbox): ")
  (pmwiki-edit (pmwiki-name-to-link
		(pmwiki-default-string name "Main.WikiSandbox"))
	       (pmwiki-loc 'base pmichaud-sandbox-url)))


(defun lyx-open (name)
  "Browse the Lyx wiki. Ex: 'Main.WikiSandbox' will open the page from
	http://wiki.lyx.org/pmwiki.php/Main/WikiSandbox"
  (interactive "sName (default Main.HomePage): ")
  (pmwiki-edit (pmwiki-name-to-link
		(pmwiki-default-string name "Main.HomePage"))
	       (pmwiki-loc 'base lyx-homepage-url)))

  
(provide 'pmwiki)
;;; pmwiki.el ends here
